#include "SiteServer.h"

#include "Sensors/TemperatureSensor.h"

#include <functional>
#include <memory>

namespace Passer {
namespace Control {

SiteServer::SiteServer(int port) {
  this->name = "Site Server";
  this->publishInterval = 0;

  this->ipAddress = "0.0.0.0";
  this->port = port;

  this->senders.push_back(this);

  SetupUDP(port, ipAddress, 0);

  Register<TemperatureSensor>((unsigned char)Thing::Type::TemperatureSensor);
}

void SiteServer::Process(RemoteParticipant *sender, ClientMsg *msg) {
  if (msg->networkId == 0) {
    std::cout << this->name << " received New Client -> " << sender->ipAddress
              << "  " << (int)sender->networkId << "\n";
    NetworkIdMsg *msg = new NetworkIdMsg(sender->networkId);
    //sender->Send(msg);
    this->Send(sender, msg);
    delete msg;
  }
}

void SiteServer::Process(RemoteParticipant *sender, NetworkIdMsg *msg) {}

void SiteServer::Process(RemoteParticipant *sender, ThingMsg *msg) {
  Thing *thing = sender->Get(msg->networkId, msg->thingId);
  if (thing == nullptr) {
    auto thingMsgProcessor = thingMsgProcessors.find(msg->thingType);
    Thing *newThing;
    if (thingMsgProcessor != thingMsgProcessors.end()) // found item
      newThing = thingMsgProcessor->second(msg->networkId, msg->thingId);
    else
      newThing = new Thing(sender, msg->networkId, msg->thingId,
                           (Thing::Type)msg->thingType);
    sender->Add(newThing);
  }
}

} // namespace Control
} // namespace Passer
